﻿/// <file>SamplePluginTestStepTypeUiProvider.cs</file>
/// 
/// <summary>The <see cref="ITestStepTypeUiProvider"/> instance that loads the test step types (with their UI metadata) from the ETB.Plugin.SamplePlugin assembly.</summary>
/// 
/// <copyright>Copyright (c) MATHEMA GmbH 2024</copyright>
/// 
/// <autor>Dominik Dinkel [DD]</autor>
/// <date>12.04.2024</date>
/// <fileversion>1.0.0.0</fileversion>
/// <revisions>
///     <revision initials="DD" date="12.04.2024" fileversion="1.0.0.0">Created.</revision>
///     <revision initials="DD" date="12.04.2024" fileversion="1.0.0.0">First finished version.</revision>
/// </revisions>

#nullable enable

#region Using

using Mathema.EmbeddedTestBench.Core.Interfaces;
using Mathema.EmbeddedTestBench.Core.Interfaces.Tests.Types;
using Mathema.EmbeddedTestBench.Core.Interfaces.UI;
using Mathema.EmbeddedTestBench.Core.Interfaces.UI.Tests;
using Mathema.EmbeddedTestBench.Core.UI.Tests;
using Mathema.EmbeddedTestBench.Plugin.SamplePlugin.Model.StepTypes;
using Mathema.EmbeddedTestBench.Plugin.SamplePlugin.UI.Resources;

using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Diagnostics.CodeAnalysis;
using System.Globalization;
using System.Linq;
using System.Reflection;

#endregion Using

namespace Mathema.EmbeddedTestBench.Plugin.SamplePlugin.UI.Metadata
{
    /// <summary>
    /// The <see cref="ITestStepTypeUiProvider"/> instance that loads the test step types (with their UI metadata) from the ETB.Plugin.SamplePlugin assembly.
    /// </summary>
    internal sealed class SamplePluginTestStepTypeUiProvider : ITestStepTypeUiProvider
    {
        #region Private static variables

        /// <summary>
        /// The assembly of ETB.Plugin.SamplePlugin.UI.
        /// </summary>
        private static readonly Assembly uiAssembly = Assembly.GetExecutingAssembly();

        #endregion Private static variables

        #region  Variables

        /// <summary>
        /// The <see cref="Dictionary{TKey, TValue}"/> where every <see cref="ITestStepTypeUiMetadata"/> and test step type from ETB.Plugin.SamplePlugin is saved together with its <see cref="Guid"/>.
        /// </summary>
        private readonly Dictionary<Guid, (ITestStepTypeUiMetadata Metadata, Type Type)> cache;

        #endregion  Variables

        #region Properties

        /// <inheritdoc/>
        public IEnumerable<ITestStepTypeUiMetadata> CachedValues
        {
            get
            {
                return cache.Values.Select(t => t.Metadata);
            }
        }

        IEnumerable<ITestStepTypeMetadata> IEnumerableCache<Guid, ITestStepTypeMetadata>.CachedValues
        {
            get
            {
                return CachedValues;
            }
        }

        #endregion Properties

        #region Constructors

        /// <summary>
        /// Constructor of the class.
        /// </summary>
        public SamplePluginTestStepTypeUiProvider()
        {
            cache = new();

            InitializeCache();
        }

        #endregion Constructors

        #region Private helper methods

        /// <summary>
        /// Fills the values for the <see cref="cache"/> dictionary.
        /// </summary>
        private void InitializeCache()
        {
            Assembly coreAssembly = typeof(DoSampleStep).Assembly;

            IEnumerable<Type> allTypeMetadata = from Type type in uiAssembly.DefinedTypes
                                                where typeof(TestStepTypeUiMetadataBase).IsAssignableFrom(type) && type.IsSealed
                                                select type;

            foreach (Type type in allTypeMetadata)
            {
                // Because our naming schema is that each metadata instance has the name of its corresponding type.
                string testStepTypeName = string.Concat("Mathema.EmbeddedTestBench.Plugin.SamplePlugin.Model.StepTypes.", type.Name.Replace("UiMetadata", string.Empty, StringComparison.Ordinal));
                Type? testStepType = coreAssembly.GetType(testStepTypeName, true, true);
                Debug.Assert(testStepType is not null, $"No test step type for metadata ${type.FullName}.");
                ConstructorInfo defaultMetadataConstructor = type.GetConstructors()[0];
                ITestStepTypeUiMetadata testStepTypeMetadata = (ITestStepTypeUiMetadata)defaultMetadataConstructor.Invoke(null);
                if (cache.TryGetValue(testStepTypeMetadata.ID, out (ITestStepTypeUiMetadata Metadata, Type Type) dontCare))
                {
                    string message = string.Format(CultureInfo.InvariantCulture, ExceptionMessages.SameGuid, testStepTypeMetadata.DisplayName, cache[testStepTypeMetadata.ID].Metadata.DisplayName);
                    throw new ArgumentException(message);
                }
                cache.Add(testStepTypeMetadata.ID, (testStepTypeMetadata, testStepType));
            }
        }

        #endregion Private helper methods

        #region Public methods

        /// <inheritdoc/>
        public bool HasType(Type value)
        {
            if (value is null)
            {
                throw new ArgumentNullException(nameof(value));
            }

            return cache.Values.Any(t => t.Type == value);
        }

        /// <inheritdoc/>
        public bool ExistsInCache(Guid key)
        {
            return cache.ContainsKey(key);
        }

        /// <inheritdoc/>
        public bool TryResolveType(Guid key, [NotNullWhen(true)] out Type? resolvedType)
        {
            if (cache.TryGetValue(key, out (ITestStepTypeUiMetadata Metadata, Type Type) value))
            {
                resolvedType = value.Type;
                return true;
            }
            else
            {
                resolvedType = null;
                return false;
            }
        }

        /// <inheritdoc/>
        public bool TryResolveKey(Type type, out Guid resolvedKey)
        {
            if (type is null)
            {
                throw new ArgumentNullException(nameof(type));
            }

            ITestStepTypeMetadata? metadata = cache.Values.FirstOrDefault(t => t.Type == type).Metadata;

            if (metadata is null)
            {
                resolvedKey = Guid.Empty;
                return false;
            }

            resolvedKey = metadata.ID;
            return true;
        }

        /// <inheritdoc/>
        public ITestStepTypeUiMetadata GetFromCache(Guid key)
        {
            if (cache.TryGetValue(key, out (ITestStepTypeUiMetadata Metadata, Type Type) value))
            {
                return value.Metadata;
            }

            throw new KeyNotFoundException(ExceptionMessages.GuidCannotBeResolved);
        }

        /// <inheritdoc/>
        ITestStepTypeMetadata ICache<Guid, ITestStepTypeMetadata>.GetFromCache(Guid key)
        {
            return GetFromCache(key);
        }

        /// <inheritdoc/>
        public IUiPlugin SourceUiPlugin(IUiPluginResolver pluginResolver)
        {
            if (pluginResolver is null)
            {
                throw new ArgumentNullException(nameof(pluginResolver));
            }

            return pluginResolver.GetUiPluginFromLoadedAssembly(uiAssembly);
        }

        /// <inheritdoc/>
        public IPlugin SourcePlugin(IPluginResolver pluginResolver)
        {
            if (pluginResolver is null)
            {
                throw new ArgumentNullException(nameof(pluginResolver));
            }

            if (pluginResolver is IUiPluginResolver uiPluginResolver)
            {
                return SourceUiPlugin(uiPluginResolver);
            }
            else
            {
                return pluginResolver.GetPluginFromLoadedAssembly(uiAssembly);
            }
        }

        #endregion Public methods
    }
}
