﻿/// <file>SamplePluginUiPlugin.cs</file>
/// 
/// <summary>The plugin for the multiplexer that contains the WPF UI elements for editing the properties in the main UI.</summary>
/// 
/// <copyright>Copyright (c) MATHEMA GmbH 2024</copyright>
/// 
/// <autor>Dominik Dinkel [DD]</autor>
/// <date>12.04.2024</date>
/// <fileversion>1.0.0.0</fileversion>
/// <revisions>
///     <revision initials="DD" date="12.04.2024" fileversion="1.0.0.0">Created.</revision>
///     <revision initials="DD" date="12.04.2024" fileversion="1.0.0.0">First finished version.</revision>
/// </revisions>

#nullable enable

#region Using

using Mathema.EmbeddedTestBench.Core.Interfaces.Hardwares.Types;
using Mathema.EmbeddedTestBench.Core.Interfaces.UI;
using Mathema.EmbeddedTestBench.Core.Interfaces.UI.PropertyEditors;
using Mathema.EmbeddedTestBench.Core.Interfaces.UI.Tests;
using Mathema.EmbeddedTestBench.Core.UI;
using Mathema.EmbeddedTestBench.Core.UI.Providers;
using Mathema.EmbeddedTestBench.Plugin.SamplePlugin.Interfaces;
using Mathema.EmbeddedTestBench.Plugin.SamplePlugin.UI.Controls;
using Mathema.EmbeddedTestBench.Plugin.SamplePlugin.UI.Helper;
using Mathema.EmbeddedTestBench.Plugin.SamplePlugin.UI.Metadata;
using Mathema.EmbeddedTestBench.Plugin.SamplePlugin.UI.Resources;

using System;
using System.Drawing;
using System.Globalization;
using System.Reflection;

#endregion Using

namespace Mathema.EmbeddedTestBench.Plugin.SamplePlugin.UI
{
    /// <summary>
    /// The ui plugin for the sample plugin that contains the WPF UI elements for editing the properties in the main UI.
    /// </summary>
    public sealed class SamplePluginUiPlugin : UiPluginBase
    {
        #region  Variables

        /// <summary>
        /// Prevents redundant calls to <see cref="Dispose(bool)"/>.
        /// </summary>
        private bool disposed;

        /// <summary>
        /// The <see cref="SamplePluginPlugin"/> for which this plugin is the UI version.
        /// </summary>
        private readonly SamplePluginPlugin basePlugin;

        /// <summary>
        /// Unique identifier for plugin
        /// </summary>
        private readonly Guid identifier = new("{d1a6eccd-83b7-4e83-8035-1bc3ea8c1f5a}");

        #endregion  Variables

        #region Properties

        /// <summary>
        /// Gets the title of this plugin that should be displayed in the Plugin-Selection-dialog.
        /// </summary>
        public override string DisplayName
        {
            get
            {
                return basePlugin.DisplayName;
            }
        }

        /// <summary>
        /// Gets the name of the company of this plugin.
        /// </summary>
        public override string Vendor
        {
            get
            {
                return basePlugin.Vendor;
            }
        }

        /// <summary>
        /// Gets the version of this plugin.
        /// </summary>
        public override Version Version { get; }

        /// <summary>
        /// Gets an icon for this plugin.
        /// </summary>
        public override Bitmap Icon { get; }


        /// <inheritdoc/>
        public override Guid Identifier
        {
            get
            {
                return identifier;
            }
        }

        #endregion Properties

        #region Constructors and IDisposable support.

        /// <summary>
        /// Constructor of the class.
        /// </summary>
        public SamplePluginUiPlugin()
        {
            basePlugin = new SamplePluginPlugin();
            Assembly assembly = Assembly.GetExecutingAssembly();
            Version = assembly.GetName().Version!;
            Icon = ImageLUT.PluginLogo;
        }

        /// <summary>
        /// Releases all resource used by the object.
        /// </summary>
        /// <param name="disposing">Set true if disposing managed resources as well as unmanaged.</param>
        protected override void Dispose(bool disposing)
        {
            base.Dispose(disposing);

            if (!disposed)
            {
                if (disposing)
                {
                    Icon.Dispose();
                    basePlugin.Dispose();
                }

                disposed = true;
            }
        }

        #endregion Constructors and IDisposable support.

        #region Public methods

        /// <inheritdoc/>
        public override void SetCulture(CultureInfo uiCulture)
        {
            ArgumentNullException.ThrowIfNull(uiCulture);

            ImageLUT.Culture = uiCulture;
            StringLUT.Culture = uiCulture;
            ExceptionMessages.Culture = uiCulture;
        }

        /// <inheritdoc/>
        public override IHardwareTypeProvider? GetHardwareTypeProvider()
        {
            return new SamplePluginHardwareTypeUiProvider();
        }

        /// <inheritdoc/>
        public override ITestStepTypeUiProvider GetTestStepTypeUiProvider()
        {
            return new SamplePluginTestStepTypeUiProvider();
        }

        /// <inheritdoc/>
        public override IDetailedResultsGuidResolver? GetResultsGuidResolver()
        {
            return new SamplePluginDetailedResultsGuidResolver();
        }

        /// <inheritdoc/>
        public override IMenuProvider GetMenuProvider()
        {
            return new SingleAssemblyMenuProvider(Assembly.GetExecutingAssembly());
        }

        /// <inheritdoc/>
        public override IControlSettings GetSpecificSettingsControl()
        {
            return new ControlSettings();
        }

        /// <inheritdoc/>
        public override IPropertyEditorProvider GetPropertyEditorProvider()
        {
            return new SingleAssemblyPropertyEditorProvider(Assembly.GetExecutingAssembly());
        }

        /// <inheritdoc/>
        public override ISamplePluginPluginSettings RetrieveSettings()
        {
            return new UiSettings();
        }
        #endregion Public methods
    }
}
