﻿/// <file>SettingsViewModel.cs</file>
/// 
/// <summary>Contains the view model of the sample settings.</summary>
/// 
/// <copyright>Copyright (c) MATHEMA GmbH 2024</copyright>
/// 
/// <autor>Dominik Dinkel [DD]</autor>
/// <date>12.04.2024</date>
/// <fileversion>1.0.0.0</fileversion>
/// <revisions>
///     <revision initials="DD" date="12.04.2024" fileversion="1.0.0.0">Created.</revision>
///     <revision initials="DD" date="12.04.2024" fileversion="1.0.0.0">First finished version.</revision>
/// </revisions>

#region Using

using Mathema.EmbeddedTestBench.Plugin.SamplePlugin.Interfaces;
using Mathema.EmbeddedTestBench.Plugin.SamplePlugin.UI.Helper;

using System;
using System.ComponentModel;

#endregion Using

namespace Mathema.EmbeddedTestBench.Plugin.SamplePlugin.UI.ViewModel
{
    /// <summary>
    /// Describes the view model used inside <see cref="Controls.ControlSettings"/>
    /// </summary>
    internal sealed class SettingsViewModel : INotifyPropertyChanged
    {
        #region  Variables

        private bool settingBool;
        private string settingString;
        private int settingNumber;

        #endregion  Variables

        #region  Events

        /// <inheritdoc/>
        public event PropertyChangedEventHandler PropertyChanged;

        #endregion  Events

        #region Properties

        /// <summary>
        /// Sample setting for a boolean
        /// </summary>
        public bool SettingBool
        {
            get
            {
                return settingBool;
            }
            set
            {

                if (value != settingBool)
                {
                    settingBool = value;
                    InvokePropertyChanged(nameof(SettingBool));
                }
            }
        }

        /// <summary>
        /// Sample setting for a string
        /// </summary>
        public string SettingString
        {
            get
            {
                return settingString;
            }
            set
            {

                if (value != settingString)
                {
                    settingString = value;
                    InvokePropertyChanged(nameof(SettingString));
                }
            }
        }

        /// <summary>
        /// Sample setting for an integer value
        /// </summary>
        public int SettingNumber
        {
            get
            {
                return settingNumber;
            }
            set
            {

                if (value != settingNumber)
                {
                    settingNumber = value;
                    InvokePropertyChanged(nameof(SettingNumber));
                }
            }
        }

        #endregion Properties

        #region Constructor

        /// <summary>
        /// Creates a new instance of this <see cref="SettingsViewModel"/> class.
        /// </summary>
        public SettingsViewModel()
        {
        }

        #endregion Constructor

        #region Public methods

        /// <summary>
        /// Initializes this instance with the values read from <paramref name="samplePluginSettings"/>.
        /// </summary>
        /// <param name="samplePluginSettings">The <see cref="ISamplePluginPluginSettings"/> instance that should be used to initialize this instance.</param>
        /// <exception cref="ArgumentNullException">If <paramref name="samplePluginSettings"/> is <see langword="null"/>.</exception>
        public void LoadFrom(ISamplePluginPluginSettings samplePluginSettings)
        {
            ArgumentNullException.ThrowIfNull(samplePluginSettings);

            settingBool = samplePluginSettings.SamplePluginSettingBool;
            settingString = samplePluginSettings.SamplePluginSettingString;
            settingNumber = samplePluginSettings.SamplePluginSettingNumber;

            InvokePropertyChanged(nameof(SettingBool));
            InvokePropertyChanged(nameof(SettingString));
            InvokePropertyChanged(nameof(SettingNumber));
        }

        /// <summary>
        /// Saves the currently entered settings in this view model in the provided <paramref name="samplePluginSettings"/>.
        /// </summary>
        /// <param name="samplePluginSettings">The <see cref="ISamplePluginPluginSettings"/> instance which values should be overwritten by the values in this instance.</param>
        /// <exception cref="ArgumentNullException">If <paramref name="samplePluginSettings"/> is <see langword="null"/>.</exception>
        public void Save(UiSettings samplePluginSettings)
        {
            ArgumentNullException.ThrowIfNull(samplePluginSettings);

            samplePluginSettings.SamplePluginSettingBool = SettingBool;
            samplePluginSettings.SamplePluginSettingString = SettingString;
            samplePluginSettings.SamplePluginSettingNumber = SettingNumber;
        }

        #endregion Public methods

        #region Private methods for event handlers

        /// <summary>
        /// Invokes the <see cref="PropertyChanged"/> event with the provided <paramref name="propertyName"/>.
        /// </summary>
        /// <param name="propertyName">The name of the property for the <see cref="PropertyChanged"/> event.</param>
        private void InvokePropertyChanged(string propertyName)
        {
            PropertyChanged?.Invoke(this, new PropertyChangedEventArgs(propertyName));
        }

        #endregion Private methods for event handlers

    }
}
