﻿/// <file>SamplePluginHardwareTypeProvider.cs</file>
/// 
/// <summary>Provides the hardware types from the SamplePlugin plugin.</summary>
/// 
/// <copyright>Copyright (c) MATHEMA GmbH 2024</copyright>
/// 
/// <autor>Dominik Dinkel [DD]</autor>
/// <date>12.04.2024</date>
/// <fileversion>1.0.0.0</fileversion>
/// <revisions>
///     <revision initials="DD" date="12.04.2024" fileversion="1.0.0.0">Created.</revision>
///     <revision initials="DD" date="12.04.2024" fileversion="1.0.0.0">First finished version.</revision>
/// </revisions>

#nullable enable

#region Using

using Mathema.EmbeddedTestBench.Core.Interfaces;
using Mathema.EmbeddedTestBench.Core.Interfaces.Hardwares.Types;
using Mathema.EmbeddedTestBench.Plugin.SamplePlugin.Resources;

using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Diagnostics.CodeAnalysis;
using System.Globalization;
using System.Linq;
using System.Reflection;

#endregion Using

namespace Mathema.EmbeddedTestBench.Plugin.SamplePlugin.Model.HardwareTypes.Metadata
{
    /// <summary>
    /// Provides the hardware types from the SamplePlugin plugin (this is only the <see cref="SampleHardwareType"/> instance).
    /// </summary>
    public class SamplePluginHardwareTypeProvider : IHardwareTypeProvider
    {
        #region Private static variables

        /// <summary>
        /// This assembly.
        /// </summary>
        private static readonly Assembly SamplePluginAssembly = Assembly.GetExecutingAssembly();

        #endregion Private static variables

        #region  Variables

        /// <summary>
        /// The <see cref="Dictionary{TKey, TValue}"/> where every <see cref="IHardwareTypeMetadata"/> and the hardware types from the SamplePlugin plugin is saved with its <see cref="Guid"/>.
        /// </summary>
        private readonly Dictionary<Guid, (IHardwareTypeMetadata Metadata, Type Type)> cache;

        #endregion  Variables

        #region Properties

        /// <inheritdoc/>
        public IEnumerable<IHardwareTypeMetadata> CachedValues
        {
            get
            {
                return cache.Values.Select(t => t.Metadata);
            }
        }

        #endregion Properties

        #region Constructors

        /// <summary>
        /// Initializes a new instance of this <see cref="SamplePluginHardwareTypeProvider"/> and fills the cache with the hardware from the SamplePlugin assembly.
        /// </summary>
        public SamplePluginHardwareTypeProvider()
        {
            cache = new();

            InitializeCache();
        }

        #endregion Constructors

        #region Private helper methods

        /// <summary>
        /// Initializes the <see cref="cache"/> dictionary.
        /// </summary>
        private void InitializeCache()
        {
            IEnumerable<Type> allTypeMetadata = from Type type in SamplePluginAssembly.DefinedTypes
                                                where typeof(IHardwareTypeMetadata).IsAssignableFrom(type) && type.IsSealed
                                                select type;

            foreach (Type type in allTypeMetadata)
            {
                // Because our naming schema is that each metadata instance has the name of its corresponding type.
                string hardwareTypeName = string.Concat("Mathema.EmbeddedTestBench.Plugin.SamplePlugin.Model.HardwareTypes.", type.Name.Replace("Metadata", string.Empty, StringComparison.Ordinal));
                Type? hardwareType = SamplePluginAssembly.GetType(hardwareTypeName, true, true);
                Debug.Assert(hardwareType is not null, $"No hardware type for metadata ${type.FullName}.");
                ConstructorInfo defaultMetadataConstructor = type.GetConstructors()[0];
                IHardwareTypeMetadata hardwareTypeMetadata = (IHardwareTypeMetadata)defaultMetadataConstructor.Invoke(null);
                if (cache.TryGetValue(hardwareTypeMetadata.ID, out (IHardwareTypeMetadata Metadata, Type Type) dontCare))
                {
                    string message = string.Format(CultureInfo.InvariantCulture, ExceptionMessages.SameGuid, hardwareTypeMetadata.DisplayName, cache[hardwareTypeMetadata.ID].Metadata.DisplayName);
                    throw new ArgumentException(message);
                }
                cache.Add(hardwareTypeMetadata.ID, (hardwareTypeMetadata, hardwareType));
            }
        }

        #endregion Private helper methods

        #region Public methods

        /// <inheritdoc/>
        public bool HasType(Type value)
        {
            ArgumentNullException.ThrowIfNull(value);

            return cache.Values.Any(t => t.Type == value);
        }

        /// <inheritdoc/>
        public bool ExistsInCache(Guid key)
        {
            return cache.ContainsKey(key);
        }

        /// <inheritdoc/>
        public bool TryResolveType(Guid key, [NotNullWhen(true)] out Type? resolvedType)
        {
            if (cache.TryGetValue(key, out (IHardwareTypeMetadata Metadata, Type Type) value))
            {
                resolvedType = value.Type;
                return true;
            }
            else
            {
                resolvedType = null;
                return false;
            }
        }

        /// <inheritdoc/>
        public bool TryResolveKey(Type type, out Guid resolvedKey)
        {
            ArgumentNullException.ThrowIfNull(type);

            IHardwareTypeMetadata? metadata = cache.Values.FirstOrDefault(t => t.Type == type).Metadata;

            if (metadata is null)
            {
                resolvedKey = Guid.Empty;
                return false;
            }

            resolvedKey = metadata.ID;
            return true;
        }

        /// <inheritdoc/>
        public IHardwareTypeMetadata GetFromCache(Guid key)
        {
            if (cache.TryGetValue(key, out (IHardwareTypeMetadata Metadata, Type Type) value))
            {
                return value.Metadata;
            }

            throw new KeyNotFoundException(ExceptionMessages.GuidCannotBeResolved);
        }

        /// <inheritdoc/>
        public virtual IPlugin SourcePlugin(IPluginResolver pluginResolver)
        {
            ArgumentNullException.ThrowIfNull(pluginResolver);

            return pluginResolver.GetPluginFromLoadedAssembly(SamplePluginAssembly);
        }

        /// <inheritdoc/>
        public IHardwareTypeMetadata DefaultHardwareTypeMetadata()
        {
            throw new KeyNotFoundException(ExceptionMessages.GuidCannotBeResolved);
        }

        #endregion Public methods
    }
}
