﻿/// <file>Mux34970AHardwareType.cs</file>
/// 
/// <summary>Represents the abstract base class for a sample hardware type that can be used in the ETB.</summary>
/// 
/// <copyright>Copyright (c) MATHEMA GmbH 2024</copyright>
/// 
/// <autor>Dominik Dinkel [DD]</autor>
/// <date>12.04.2024</date>
/// <fileversion>1.0.0.0</fileversion>
/// <revisions>
///     <revision initials="DD" date="12.04.2024" fileversion="1.0.0.0">Created.</revision>
///     <revision initials="DD" date="12.04.2024" fileversion="1.0.0.0">First finished version.</revision>
/// </revisions>

#region Using

using Mathema.EmbeddedTestBench.Core.Attributes.Hardwares;
using Mathema.EmbeddedTestBench.Core.Interfaces.Export;
using Mathema.EmbeddedTestBench.Core.Interfaces.Hardwares;
using Mathema.EmbeddedTestBench.Core.Interfaces.Hardwares.Types;
using Mathema.EmbeddedTestBench.Core.Model.Hardwares.Types;
using Mathema.EmbeddedTestBench.Plugin.SamplePlugin.Interfaces;
using Mathema.EmbeddedTestBench.Plugin.SamplePlugin.Resources;

using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.IO.Ports;
using System.Runtime.Serialization;
using System.Threading;
using System.Xml;

#endregion Using

namespace Mathema.EmbeddedTestBench.Plugin.SamplePlugin.Model.HardwareTypes
{
    /// <summary>
    /// Represents the abstract base class for a sample hardware type that can be used in the ETB.
    /// </summary>
    [HardwareType("{AA8EB109-F859-4957-859B-7A8229D1CC9B}",
        DisplayNameResourceType = typeof(StringLUT),
        DisplayNameResourceKey = "HWTYPE_Sample_DisplayName",
        DescriptionResourceKey = "HWTYPE_Sample_Description"
    )]
    [DataContract(IsReference = true)]
    public sealed class SampleHardwareType : HardwareTypeBase, ISampleHardwareType
    {
        #region Private const variables

        /// <summary>
        /// The timeout for the <see cref="SerialPort.ReadTimeout"/> property in milliseconds.
        /// </summary>
        private const int ReadTimeoutMilliseconds = 3000;

        #endregion Private const variables

        #region Variables

        /// <summary>
        /// The name of the com port where the sample is connected to.
        /// </summary>
        private string comPortName = string.Empty;

        /// <summary>
        /// SerialPort for Communication
        /// </summary>
        private SerialPort comPort;

        /// <summary>
        /// Prevents redundant calls to <see cref="Dispose(bool)"/>.
        /// </summary>
        private bool disposed;

        #endregion Variables

        #region Properties


        /// <inheritdoc/>
        [DataMember(EmitDefaultValue = false)]
        public string ComPortName
        {
            get
            {
                return comPortName;
            }

            set
            {
                if (value is null)
                {
                    throw new ArgumentNullException(nameof(value));
                }

                if (comPort != null && comPort.IsOpen)
                {
                    throw new InvalidOperationException();
                }

                SetProperty(ref comPortName, value);
            }
        }

        /// <inheritdoc/>
        public override bool IsDirty
        {
            get
            {
                return base.IsDirty;
            }
        }

        #endregion Properties

        #region Constructors and IDisposable support

        /// <summary>
        /// Creates a new instance of this <see cref="SampleHardwareType"/> class that resides in the provided <paramref name="hardwareContainer"/> instance.
        /// </summary>
        /// <param name="hardwareContainer">The parent hardware container where this hardware type instance is.</param>
        /// <exception cref="ArgumentNullException">If <paramref name="hardwareContainer"/> is <see langword="null"/>.</exception>
        public SampleHardwareType(IHardware hardwareContainer) : base(hardwareContainer) { }

        /// <summary>
        /// Creates a new instance of this <see cref="SampleHardwareType"/> class that is in the provided <paramref name="hardwareContainer"/> that contains the provided  <paramref name="comPortName"/>.
        /// </summary>
        /// <param name="hardwareContainer">The parent hardware container where this hardware type instance is.</param>
        /// <param name="comPortName">The name of the ComPort where the sample hw is connected to.</param>
        /// <exception cref="ArgumentNullException">If <paramref name="hardwareContainer"/> or <paramref name="comPortName"/> is <see langword="null"/>.</exception>
        public SampleHardwareType(IHardware hardwareContainer, string comPortName) : this(hardwareContainer)
        {
            this.comPortName = comPortName ?? throw new ArgumentNullException(nameof(comPortName));
        }

        /// <summary>
        /// Releases all resource used by the object.
        /// </summary>
        /// <param name="disposing">Set true if disposing managed resources as well as unmanaged.</param>
        protected override void Dispose(bool disposing)
        {
            base.Dispose(disposing);

            if (!disposed)
            {
                if (disposing)
                {
                    // Free other state (managed objects).
                    if (comPort != null)
                    {
                        if (comPort.IsOpen)
                        {
                            comPort.Close();
                        }

                        comPort.Dispose();
                    }
                }

                // Free your own state (unmanaged objects).
                // Set large fields to null.

                disposed = true;
            }
        }

        #endregion Constructors and IDisposable support

        #region Public methods

        /// <inheritdoc/>
        public void OpenComPort()
        {
            comPort ??= new SerialPort(ComPortName, 115200, Parity.None, 8, StopBits.One)
            {
                Handshake = Handshake.XOnXOff,
                ReadTimeout = ReadTimeoutMilliseconds
            };

            if (!comPort.IsOpen)
            {
                Trace.TraceWarning($"Open ComPort for Sample Hardware: {comPort.PortName}");
                comPort.Open();
            }
        }

        /// <inheritdoc/>
        public void CloseComPort()
        {
            if (comPort != null && comPort.IsOpen)
            {
                Trace.TraceWarning($"Close ComPort for Sample Hardware: {comPort.PortName}");
                comPort.Close();
                comPort.Dispose();
                comPort = null;
            }
        }

        /// <inheritdoc/>
        public void WriteData(string data)
        {
            ArgumentNullException.ThrowIfNull(comPort);

            comPort.WriteLine(data);
        }

        /// <inheritdoc/>
        public override void Initialize()
        {
            OpenComPort();
        }

        /// <inheritdoc/>
        public override void Shutdown()
        {
            CloseComPort();
        }

        /// <inheritdoc/>
        public override HardwareComPortMapper CreateMapperIfAvailable()
        {
            // In case, no COM part had been specified, we use an empty string for the mapper.
            return new HardwareComPortMapper(HardwareContainer.ID, HardwareContainer.DisplayName, comPortName ?? string.Empty);
        }

        /// <inheritdoc/>
        public override void OverwriteFromMapper(HardwareComPortMapper mapper)
        {
            if (mapper is null)
            {
                throw new ArgumentNullException(nameof(mapper));
            }

            if (HardwareContainer.ID == mapper.HardwareID)
            {
                comPortName = mapper.ComPortName;
            }
        }

        /// <inheritdoc/>
        public override IHardwareType CreateDeepCopy(IHardware parent)
        {
            return new SampleHardwareType(parent, ComPortName);
        }

        /// <inheritdoc/>
        public override void WriteToXml(IExportElementDecider decider, ICollection<KeyValuePair<string, string>> parameters, IExporter exporter, CancellationToken cancellationToken)
        {
            ArgumentNullException.ThrowIfNull(exporter, nameof(exporter));

            base.WriteToXml(decider, parameters, exporter, cancellationToken);

            exporter.CreateHardwareKeyValueCaptionEntries(nameof(ComPortName), StringLUT.HWTYPE_PROPERTY_ComPortName_DisplayName, ComPortName, cancellationToken);
        }

        #endregion Public methods
    }

}
