﻿/// <file>DoSampleStep.cs</file>
/// 
/// <summary>Simple sample test step without a detail result view.</summary>
/// 
/// <copyright>Copyright (c) MATHEMA GmbH 2024</copyright>
/// 
/// <autor>Dominik Dinkel [DD]</autor>
/// <date>12.04.2024</date>
/// <fileversion>1.0.0.0</fileversion>
/// <revisions>
///     <revision initials="DD" date="12.04.2024" fileversion="1.0.0.0">Created.</revision>
///     <revision initials="DD" date="12.04.2024" fileversion="1.0.0.0">First finished version.</revision>
/// </revisions>

#region Using

using Mathema.EmbeddedTestBench.Core.Enums.Enums;
using Mathema.EmbeddedTestBench.Core.Interfaces.Export;
using Mathema.EmbeddedTestBench.Core.Interfaces.Hardwares.Types;
using Mathema.EmbeddedTestBench.Core.Interfaces.Tests;
using Mathema.EmbeddedTestBench.Core.Interfaces.Tests.Steps;
using Mathema.EmbeddedTestBench.Core.Interfaces.Tests.Types;
using Mathema.EmbeddedTestBench.Core.Model.Tests;
using Mathema.EmbeddedTestBench.Core.Model.Tests.Types;
using Mathema.EmbeddedTestBench.Plugin.SamplePlugin.Interfaces;
using Mathema.EmbeddedTestBench.Plugin.SamplePlugin.Resources;

using System;
using System.Collections.Generic;
using System.Globalization;
using System.Runtime.Serialization;
using System.Threading;
using System.Threading.Tasks;

#endregion Using

namespace Mathema.EmbeddedTestBench.Plugin.SamplePlugin.Model.StepTypes
{
    /// <summary>
    /// Represents a simple teststep with a custom property editor.
    /// </summary>
    [DataContract(IsReference = true)]
    public sealed class DoSampleStep : TestStepTypeBase
    {
        #region Variables

        private ISampleHardwareType sampleHardware;
        private int doNumber;
        private string demoString;

        #endregion Variables

        #region Properties

        /// <summary>
        /// Gets or sets the sample hardware type that is used in this test step type.
        /// </summary>
        [DataMember]
        public ISampleHardwareType SampleHardware
        {
            get
            {
                return sampleHardware;
            }

            set
            {
                SetProperty(ref sampleHardware, value);
            }
        }

        /// <summary>
        /// Sample property for an integer value
        /// </summary>
        [DataMember]
        public int DoNumber
        {
            get
            {
                return doNumber;
            }
            set
            {
                SetProperty(ref doNumber, value);
            }
        }

        /// <summary>
        /// Sample property for a string
        /// </summary>
        [DataMember]
        public string DemoString
        {
            get
            {
                return demoString;
            }
            set
            {
                SetProperty(ref demoString, value);
            }
        }

        /// <inheritdoc/>
        public override bool IsDirty
        {
            get
            {
                return base.IsDirty;
            }
        }

        #endregion Properties

        #region Constructors

        /// <summary>
        /// Creates a new and empty instance of this <see cref="DoSampleStep"/> class.
        /// </summary>
        /// <param name="testStep">The parent <see cref="ITestStep"/> instance.</param>
        /// <exception cref="ArgumentNullException">If <paramref name="testStep"/> is <see langword="null"/>.</exception>
        public DoSampleStep(ITestStep testStep) : base(testStep, StringLUT.STEP_DoSampleStep_DisplayName)
        {
            DoNumber = -1;
            DemoString = "demodaten";
        }

        /// <summary>
        /// Private constructor for the <see cref="CreateDeepCopy(ITestStep)"/> method to copy the provided <paramref name="doSampleStep"/>..
        /// </summary>
        /// <param name="testStep">The parent <see cref="ITestStep"/> instance.</param>
        /// <param name="doSampleStep">The <see cref="DoSampleStep"/> instance for the copy.</param>
        private DoSampleStep(ITestStep testStep, DoSampleStep doSampleStep) : base(testStep, StringLUT.STEP_DoSampleStep_DisplayName)
        {
            SampleHardware = doSampleStep.SampleHardware;
            DoNumber = doSampleStep.DoNumber;
            DemoString = doSampleStep.demoString;
        }

        #endregion Constructors

        #region Private helper methods

        /// <inheritdoc/>
        protected override void RemoveDirtyStateFromChildElements()
        {
            base.RemoveDirtyStateFromChildElements();
        }

        /// <inheritdoc/>
        protected override async Task ExecuteInternal(ITestExecutionContext context, IProgress<ITestExecutionProgress> progress, CancellationToken cancellationToken)
        {
            ArgumentNullException.ThrowIfNull(progress);

            if (SampleHardware is null)
            {
                throw new InvalidOperationException(StringLUT.EXEC_NoHardware);
            }

            //Do Stuff of the TestStep
            await doWork(cancellationToken).ConfigureAwait(false);

            if (doNumber > 100)
            {
                progress.Report(new TestExecutionProgress(1, TestExecutionState.Passed, demoString));
            }
            else
            {
                progress.Report(new TestExecutionProgress(1, TestExecutionState.Failed, demoString));
            }

        }

        /// <summary>
        /// Sample for an action within a steptype
        /// </summary>
        /// <param name="cancellationToken"><see cref="CancellationToken"/> to allow the abortion of the steptype</param>
        /// <returns></returns>
        private async Task doWork(CancellationToken cancellationToken)
        {
            await Task.Delay(500, cancellationToken).ConfigureAwait(false);
            SampleHardware.WriteData("Writing Test Data");
        }

        #endregion Private helper methods

        #region Public methods

        /// <inheritdoc/>
        public override ITestStepType CreateDeepCopy(ITestStep parent)
        {
            ArgumentNullException.ThrowIfNull(parent);

            return new DoSampleStep(parent, this);
        }

        /// <inheritdoc/>
        public override IHardwareType GetReferencedHardwareType()
        {
            return sampleHardware;
        }

        /// <inheritdoc/>
        public override string GetPropertyDisplayValue(ITestStepPropertyMetadata propMetadata)
        {
            ArgumentNullException.ThrowIfNull(propMetadata);
            string result = propMetadata.Name switch
            {
                nameof(SampleHardware) => SampleHardware.HardwareContainer.DisplayName,
                nameof(DemoString) => DemoString,
                nameof(DoNumber) => DoNumber.ToString(CultureInfo.InvariantCulture),
                _ => base.GetPropertyDisplayValue(propMetadata)
            };

            return result;
        }

        /// <inheritdoc/>
        public override void WriteToXml(IExportElementDecider decider, ICollection<KeyValuePair<string, string>> parameters, IExporter exporter, CancellationToken cancellationToken)
        {
            ArgumentNullException.ThrowIfNull(decider);
            ArgumentNullException.ThrowIfNull(exporter);

            cancellationToken.ThrowIfCancellationRequested();

            exporter.CreateStepPropertyKeyValueCaptionEntries(nameof(SampleHardware), StringLUT.Hardware, SampleHardware?.HardwareContainer?.DisplayName, cancellationToken);
            exporter.CreateStepPropertyKeyValueCaptionEntries(nameof(DoNumber), StringLUT.DoSampleNumber_DisplayName, DoNumber.ToString(CultureInfo.InvariantCulture), cancellationToken);
            exporter.CreateStepPropertyKeyValueCaptionEntries(nameof(DemoString), StringLUT.StuffStrings, DemoString, cancellationToken);
        }

        #endregion Public methods

    }
}
