﻿/// <file>SamplePluginTestStepTypeProvider.cs</file>
/// 
/// <summary>The <see cref="ITestStepTypeProvider"/> instance that loads the test step types from the ETB.Plugin.SamplePlugin-assembly.</summary>
/// 
/// <copyright>Copyright (c) MATHEMA GmbH 2024</copyright>
/// 
/// <autor>Dominik Dinkel [DD]</autor>
/// <date>12.04.2024</date>
/// <fileversion>1.0.0.0</fileversion>
/// <revisions>
///     <revision initials="DD" date="12.04.2024" fileversion="1.0.0.0">Created.</revision>
///     <revision initials="DD" date="12.04.2024" fileversion="1.0.0.0">First finished version.</revision>
/// </revisions>

#nullable enable

#region Using

using Mathema.EmbeddedTestBench.Core.Interfaces;
using Mathema.EmbeddedTestBench.Core.Interfaces.Tests.Types;
using Mathema.EmbeddedTestBench.Core.Model.Tests.Types;
using Mathema.EmbeddedTestBench.Plugin.SamplePlugin.Resources;

using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Diagnostics.CodeAnalysis;
using System.Linq;
using System.Reflection;

#endregion Using

namespace Mathema.EmbeddedTestBench.Plugin.SamplePlugin.Model.StepTypes.Metadata
{
    /// <summary>
    /// The <see cref="ITestStepTypeProvider"/> instance that loads the test step types from the ETB.Plugin.SamplePlugin-assembly.
    /// </summary>
    public sealed class SamplePluginTestStepTypeProvider : ITestStepTypeProvider
    {
        #region Private static variables

        /// <summary>
        /// The assembly of ETB.Plugin.SamplePlugin.
        /// </summary>
        private static readonly Assembly coreAssembly = Assembly.GetExecutingAssembly();

        #endregion Private static variables

        #region  Variables

        /// <summary>
        /// The <see cref="Dictionary{TKey, TValue}"/> where every <see cref="ITestStepTypeMetadata"/> and test step type from ETB.Plugin.SamplePlugin is saved together with its <see cref="Guid"/>.
        /// </summary>
        private readonly Dictionary<Guid, (ITestStepTypeMetadata Metadata, Type Type)> cache;

        #endregion  Variables

        #region Properties

        /// <inheritdoc/>
        public IEnumerable<ITestStepTypeMetadata> CachedValues
        {
            get
            {
                return cache.Values.Select(t => t.Metadata);
            }
        }

        #endregion Properties

        #region Constructors

        /// <summary>
        /// Creates a new instance of this <see cref="SamplePluginTestStepTypeProvider"/> class.
        /// </summary>
        public SamplePluginTestStepTypeProvider()
        {
            cache = new();

            InitializeCache();
        }

        #endregion Constructors

        #region Private helper methods

        /// <summary>
        /// Fills the values for the <see cref="cache"/> dictionary.
        /// </summary>
        private void InitializeCache()
        {
            IEnumerable<Type> allTypeMetadata = from Type type in coreAssembly.DefinedTypes
                                                where typeof(TestStepTypeMetadataBase).IsAssignableFrom(type) && type.IsSealed
                                                select type;

            foreach (Type type in allTypeMetadata)
            {
                // Because our naming schema is that each metadata instance has the name of its corresponding type.
                string testStepTypeName = string.Concat("Mathema.EmbeddedTestBench.Plugin.SamplePlugin.Model.StepTypes.", type.Name.Replace("Metadata", string.Empty, StringComparison.Ordinal));
                Type? testStepType = coreAssembly.GetType(testStepTypeName, true, true);
                Debug.Assert(testStepType is not null, $"No test step type for metadata ${type.FullName}.");
                ConstructorInfo defaultMetadataConstructor = type.GetConstructors()[0];
                ITestStepTypeMetadata testStepTypeMetadata = (ITestStepTypeMetadata)defaultMetadataConstructor.Invoke(null);
                cache.Add(testStepTypeMetadata.ID, (testStepTypeMetadata, testStepType));
            }
        }

        #endregion Private helper methods

        #region Public methods

        /// <inheritdoc/>
        public bool HasType(Type value)
        {
            ArgumentNullException.ThrowIfNull(value);

            return cache.Values.Any(t => t.Type == value);
        }

        /// <inheritdoc/>
        public bool ExistsInCache(Guid key)
        {
            return cache.ContainsKey(key);
        }

        /// <inheritdoc/>
        public bool TryResolveType(Guid key, [NotNullWhen(true)] out Type? resolvedType)
        {
            if (cache.TryGetValue(key, out (ITestStepTypeMetadata Metadata, Type Type) value))
            {
                resolvedType = value.Type;
                return true;
            }
            else
            {
                resolvedType = null;
                return false;
            }
        }

        /// <inheritdoc/>
        public bool TryResolveKey(Type type, out Guid resolvedKey)
        {
            ArgumentNullException.ThrowIfNull(type);

            ITestStepTypeMetadata? metadata = cache.Values.FirstOrDefault(t => t.Type == type).Metadata;

            if (metadata is null)
            {
                resolvedKey = Guid.Empty;
                return false;
            }

            resolvedKey = metadata.ID;
            return true;
        }

        /// <inheritdoc/>
        public ITestStepTypeMetadata GetFromCache(Guid key)
        {
            if (cache.TryGetValue(key, out (ITestStepTypeMetadata Metadata, Type Type) value))
            {
                return value.Metadata;
            }

            throw new KeyNotFoundException(ExceptionMessages.GuidCannotBeResolved);
        }

        /// <inheritdoc/>
        public IPlugin SourcePlugin(IPluginResolver pluginResolver)
        {
            ArgumentNullException.ThrowIfNull(pluginResolver);

            return pluginResolver.GetPluginFromLoadedAssembly(coreAssembly);
        }

        #endregion Public methods
    }
}
